import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis,
  timeFormat
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// Formats the value to render on Y-axis
const yAxisTickFormat = number =>
    format('.5s')(number)

// Formats the value to render on X-axis
const xAxisTickFormat = timeFormat("%b");

// Renders the Area Chart
const render = data => {
  
  // X-axis = Date
  const xValue = d => d.Date;
  const xAxisLabel = 'Month';
  
  // Y-axis = ening bid price
  const yValue = d => d.BO;
  const yAxisLabel = 'Opening bid';
  
  const margin = { top: 10, right: 40, bottom: 88, left: 150 };
  
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
// Create a scale for X axis based on the xValue from the data
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
// Create a scale for Y axis based on the yValue from the data
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
 // The position of SVG on the graph
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15)
  	.tickFormat(xAxisTickFormat);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(yAxisTickFormat);
  
  // Renders Y-axis
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -93)
      .attr('x', -innerHeight / 2)
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  // Renders X-axis
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .text(xAxisLabel);
 // Generates the line based on the xValues and yValues
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
 // Generates the area under the curve
  const areaGenerator = area()
  	.x(d => xScale(xValue(d)))
  	.y0(innerHeight)
  	.y1(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
  g.append('path')
  	.attr('class', 'line-chart')
  	.attr('d', lineGenerator(data));
  
  g.append('path')
  	.attr('class', 'area-chart')
  	.attr('d', areaGenerator(data));
  
 
};

// Data set URL
csv('https://gist.githubusercontent.com/17761A04F1/6dc492901e67cdd989fd05cc6c5ce1ea/raw/f75fb2b6afefa3b7ee61efd3941d6601246eb74c/data.csv')
  .then(data => {
    data.forEach(d => {
      d.BO = +d.BO;
      d.Date = new Date(d.Date);
    });
    render(data);
  });